import 'package:collection/collection.dart';
import 'package:dash_deck/models/code_snippet.model.dart';
import 'package:dash_deck/models/slide_options.model.dart';
import 'package:yaml/yaml.dart';

/// Document containing the original `value`, front matter `data` and `content`.
class SlideData {
  const SlideData({
    required this.name,
    this.content,
    this.options = const SlideOptions(),
    List<String>? snippets,
  }) : _snippets = snippets ?? const [];

  final String name;
  final String? content;

  final List<String> _snippets;

  List<CodeSnippet> get snippets {
    return _snippets
        .mapIndexed((index, snippet) => CodeSnippet(
              name: '${name}_code_snippet_$index',
              source: snippet,
            ))
        .toList();
  }

  /// The parsed YAML front matter as a [YamlMap].
  final SlideOptions options;
}
